@@ -1,9 +1,11 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[1]:
+# In[64]:
 
 
+version = '0.29'
+
 import re
 import tempfile
 import os
@@ -25,7 +27,7 @@ get_ipython().run_cell_magic('javascript', '', 'var nb = IPython.notebook;\nvar
 
 def isnotebook():
     try:
-        return file_name == 'lx.ipynb'
+        return get_ipython().__class__.__name__ == 'ZMQInteractiveShell'
     except:
         pass
     return False
@@ -34,14 +36,25 @@ def isnotebook():
 # In[4]:
 
 
+def is_testing():
+    try:
+        return file_name == 'lx.ipynb'
+    except:
+        pass
+    return False
+
+
+# In[5]:
+
+
 def jprint(*args):
-    if isnotebook():
+    if is_testing():
         print(*args)
 
 
 # # Requiremtns
 
-# In[5]:
+# In[6]:
 
 
 requirements = []
@@ -61,10 +74,83 @@ from PIL import Image
 requirements.append('urllib3')
 from urllib.request import urlopen
 
+requirements.append('ipython')
+from IPython.core.display import display, HTML
+
+
+# # Lists
+
+# In[7]:
+
+
+def listsGetPairsFromTwoLists(list1, list2):
+    return list(zip(list1, list2))
+
+
+# In[8]:
+
+
+if is_testing():
+    listAbcd = ['A', 'B', 'C', 'D']
+    list1234 = ['1', '2', '3', '4']
+
+    print(listsGetPairsFromTwoLists(listAbcd, list1234))
+
+
+# In[9]:
+
+
+def listSequentialOverlappingPairs(l):
+    return list(zip(l[:-1], l[1:]))
+
+
+# In[10]:
+
+
+if is_testing:
+    list1234 = ['1', '2', '3', '4']
+    print(listSequentialOverlappingPairs(list1234))
+
+
+# # List of Strings
+
+# In[11]:
+
+
+regex_cache = {}
+def sListFilter(l, re_include=".*", re_exclude='^\b$'):
+    regex_cache[re_include] = regex_cache.get(re_include, re.compile(re_include))
+    regex_cache[re_exclude] = regex_cache.get(re_exclude, re.compile(re_exclude))
+    return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
+
+
+# In[12]:
+
+
+jprint(sListFilter(['good x', 'other good x', 'without the letter before y']))
+
+
+# In[13]:
+
+
+jprint(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
+
+
+# In[14]:
+
+
+jprint(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
+
+
+# In[15]:
+
+
+jprint(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
+
 
 # # Text files
 
-# In[6]:
+# In[16]:
 
 
 def txtread(path):
@@ -80,52 +166,61 @@ def txtwrite(path, txt):
         return f.write(txt)
 
 
-# In[7]:
+# In[17]:
 
 
-if isnotebook():
+if is_testing():
     print(txtread('https://arxiv.org'))
 
 
 # # List of Strings
 
-# In[8]:
+# In[18]:
 
 
 regex_cache = {}
-def sListFilter(l, re_include=".*", re_exclude='^\b$'):
+def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
     regex_cache[re_include] = regex_cache.get(re_include, re.compile(re_include))
     regex_cache[re_exclude] = regex_cache.get(re_exclude, re.compile(re_exclude))
-    return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
+    return [e for e in listOfStrings if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[9]:
+# ### Examples
+
+# In[19]:
 
 
 jprint(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[10]:
+# In[20]:
 
 
 jprint(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[11]:
+# In[21]:
 
 
 jprint(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[12]:
+# In[22]:
 
 
 jprint(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
+# In[23]:
+
+
+if is_testing():
+    print(txtread('https://arxiv.org'))
+
+
 # # Images
 
-# In[13]:
+# In[24]:
 
 
 def imread(path):
@@ -136,7 +231,7 @@ def imread(path):
         img = cv2.imdecode(raw, cv2.IMREAD_COLOR)
     elif os.path.isfile(path):
         img = cv2.imread(path, cv2.IMREAD_COLOR)
-    return img[:, :, [2, 1, 0]]
+    return img[:, :, [2, 1, 0]] # This converts the cv2 colors to RGB
 
 def imsread(paths):
     assert isinstance(paths, list) or isinstance(paths, tuple), type(paths)
@@ -153,7 +248,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[14]:
+# In[25]:
 
 
 def imshow(array):
@@ -162,23 +257,25 @@ def imshow(array):
         display(Image.fromarray(array))
     else:
         cv2.imshow('img', array[:, :, [2, 1, 0]])
+        cv2.waitKey()
 
 
-# In[15]:
+# In[26]:
 
 
-if isnotebook():
-    imshow(imread('http://via.placeholder.com/70.png'))
+if is_testing():
+    img = imread('http://via.placeholder.com/70.png')
+    imshow(img)
 
 
-# In[16]:
+# In[27]:
 
 
-if isnotebook():
+if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[17]:
+# In[28]:
 
 
 def imscaleNN(img, s):
@@ -188,16 +285,17 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[18]:
+# In[29]:
 
 
-if isnotebook():
+if is_testing():
     with tempfile.TemporaryDirectory() as d:
         img = (255 * np.random.rand(20, 20, 3)).astype(np.uint8)
         img_path = os.path.join(d, 'test.png')
         imwrite(img_path, img)
         img_read = imread(img_path)
-        out = imjoin([impad(img, right=5), img_read])
+        
+        out = imjoin([impad(img, right=5), img_read]) # join Images side by side
 
         out_large_pixaleted = imscaleNN(out, 8)
         imshow(out_large_pixaleted)
@@ -206,33 +304,33 @@ if isnotebook():
         imshow(out_large_interbic)
 
 
-# In[19]:
+# In[30]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[20]:
+# In[31]:
 
 
-if isnotebook(): imshow(imNewWhite(10, 500) - 30)
+if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[21]:
+# In[32]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[22]:
+# In[33]:
 
 
-if isnotebook(): imshow(imNewBlack(10, 500))
+if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[23]:
+# In[34]:
 
 
 def imAddNoiseGauss(img, std):
@@ -241,10 +339,10 @@ def imAddNoiseGauss(img, std):
     noisy = (np.clip(img.astype(np.float) + noise.astype(np.float), 0, 255)).astype(np.uint8)
     return noisy
 
-if isnotebook(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
+if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[24]:
+# In[35]:
 
 
 def imJpgDegradation(img, quality):
@@ -254,10 +352,10 @@ def imJpgDegradation(img, quality):
     decimg = cv2.imdecode(encimg, 1)
     return decimg
 
-if isnotebook(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
+if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[25]:
+# In[36]:
 
 
 def imCropCenter(img, size):
@@ -272,38 +370,39 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[26]:
+# In[37]:
 
 
-if isnotebook(): 
+if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[27]:
+# In[38]:
 
 
-if isnotebook(): 
+if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[28]:
+# In[39]:
 
 
-if isnotebook(): 
+if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[29]:
+# In[40]:
 
 
-if isnotebook(): 
+if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[30]:
+# In[41]:
 
 
 def imGallery(imgs, pad=0):
+    imgs = [imread(img) for img in imgs] # load if path or url
     n = len(imgs)
     nw = int(np.ceil(np.sqrt(n)))
     nh = int(np.ceil(n / nw))
@@ -330,77 +429,77 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[31]:
+# In[42]:
 
 
-if isnotebook():
+if is_testing():
     # List of images from back to white
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[32]:
+# In[43]:
 
 
-if isnotebook():
+if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[33]:
+# In[44]:
 
 
-if isnotebook():
+if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[34]:
+# In[45]:
 
 
-if isnotebook():
+if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[35]:
+# In[46]:
 
 
-if isnotebook():
+if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[36]:
+# In[47]:
 
 
-if isnotebook():
+if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[37]:
+# In[48]:
 
 
-if isnotebook():
+if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[38]:
+# In[49]:
 
 
-if isnotebook():
+if is_testing():
     img40x20 = imread('http://via.placeholder.com/40x20.png')
     imshow(imGallery([img40x20, img40x20, img40x20], pad=1))
 
 
 # # Files
 
-# In[39]:
+# In[50]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[40]:
+# In[51]:
 
 
-if isnotebook():
+if is_testing():
     with tempfile.TemporaryDirectory() as d:
         listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
         
@@ -426,21 +525,36 @@ if isnotebook():
         imshow(imGallery(imgs))
 
 
-# In[ ]:
+# # Jupyter
+
+# In[52]:
 
 
+def jnImageRenderingPixelated():
+    """Do not interpolate images"""
+    display(HTML('<style type = text/css> img { image-rendering: pixelated; } </style>'))
+    
+def jnImageRenderingAuto():
+    """Interpolate images"""
+    display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-
-# In[ ]:
+# In[53]:
 
 
+def jnHtmlRow(strList, width=100):
+    to_col = lambda x: '<td style="text-align:center">' + x + '</td>'
+    display(HTML('<table width="{width}%"><tr>{cols}</tr></table>'.format(width=width, cols="".join(map(to_col, strList)))))
 
+if is_testing():
+    jnHtmlRow(['text', 'looooooooooooong text', 'text'])
+    jnHtmlRow(['text', 'looooooooooooong text', 'text'], width=50)
+    
 
 
 # # Shell
 
-# In[52]:
+# In[54]:
 
 
 def shell(cmd):
@@ -450,20 +564,30 @@ def shell(cmd):
 
 # # Package Files
 
-# In[41]:
+# In[55]:
 
 
-package_files = {}
+if is_testing():
+    package_files = {}
 
 
-# In[42]:
+# In[56]:
 
 
-package_files['setup.py'] = r"""from distutils.core import setup
+def shell(cmd):
+    result = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)
+    return result.stdout.decode()
+
+
+# In[57]:
+
+
+if is_testing():
+    package_files['setup.py'] = r"""from distutils.core import setup
 setup(
   name = 'lx',         # How you named your package folder (MyLib)
   packages = ['lx'],   # Chose the same as "name"
-  version = '0.21',      # Start with a small number and increase it with every change you make
+  version = '{version}',      # Start with a small number and increase it with every change you make
   license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
   description = 'lx',   # Give a short description about your library
   author = 'lx',                   # Type in your name
@@ -484,22 +608,24 @@ setup(
     'Programming Language :: Python :: 3.5',
     'Programming Language :: Python :: 3.6',
   ],
-)""".format(requirements=", ".join(["'" + r + "'" for r in requirements]))
+)""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[43]:
+# In[58]:
 
 
-package_files['setup.cfg'] = r"""# Inside of setup.cfg
+if is_testing():
+    package_files['setup.cfg'] = r"""# Inside of setup.cfg
 [metadata]
 description-file = README.md
 """
 
 
-# In[44]:
+# In[59]:
 
 
-package_files['LICENSE.txt'] = r"""MIT License
+if is_testing():
+    package_files['LICENSE.txt'] = r"""MIT License
 Copyright (c) 2018 YOUR NAME
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -518,24 +644,42 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[45]:
+# In[60]:
 
 
-package_files['README.md'] = ""
+if is_testing():
+    package_files['README.md'] = ""
 
 
-# In[47]:
+# In[61]:
 
 
-if isnotebook():
+if is_testing():
     get_ipython().system(' jupyter nbconvert --to script lx.ipynb')
     package_files['lx/__init__.py'] = txtread("lx.py")
 
 
-# In[51]:
+# In[ ]:
 
 
-if isnotebook():
+
+
+
+# In[62]:
+
+
+if is_testing():
+    package_files['MANIFEST'] = r"""# file GENERATED by distutils, do NOT edit
+setup.cfg
+setup.py
+lx/__init__.py
+"""
+
+
+# In[65]:
+
+
+if is_testing():
     with tempfile.TemporaryDirectory() as d:
         for key, value in package_files.items():
             txtwrite(os.path.join(d, key), value)
@@ -569,3 +713,9 @@ if isnotebook():
 
 
 
+
+# In[ ]:
+
+
+
+
